#!/bin/bash
## This script assumes to be run by the root user (with /sbin in the path).

set -e

# get the root of the git repository (requires git to be installed)
GIT_ROOT=$(git rev-parse --show-toplevel)
cd $GIT_ROOT

source installation/00env.sh

image_hash=$(tail -n1 installation/images-sha512.list | cut -d' ' -f1)
shacache_url=https://shacache.nxdcdn.com/

get_partition_path () {
	# Return the path of the desired partition.
	# Exit and display an error if it finds 0 or several results.
	#
	# This function should be called with exactly one argument: the desired partition type.
	# Its behavior is undefined otherwise.

	desired_partitions=$(fdisk -l | grep "$1")

	desired_partition_count=$(echo -e "$desired_partitions" | wc | awk '{print $1}')
	if [ -z "$desired_partitions" ]; then
		echo -e "No partition of type $1 (shouldbe 1). Exiting."
		exit
	elif [ "$desired_partition_count" -ne 1 ]; then
		echo -e "$desired_partition_count partitions of type $1 (should be 1). Exiting."
		exit
	else
		echo -e "$desired_partitions" | awk '{print $1}'
	fi
}

get_partition_id () {
	desired_partition_path=$(get_partition_path "$1")
	echo -e "$(findmnt -fn -o UUID "$desired_partition_path")"
}


# Get information about the partition layout
root_partition_type="Linux filesystem"
efi_partition_type="EFI System"
EFI_PARTITION_MOUNT_POINT=$(get_partition_path "$efi_partition_type")
ROOT_PARTITION_ID=$(get_partition_id "$root_partition_type")
default_bootnum=100
# needed by efibootmgr
RELATIVE_EFI_IMAGE_DIR=EFI/Linux

# NETWORKCACHE DOWNLOADING METHOD
tmp_image_name=image_tmp.efi
networkcache-download --config ../tls-access-to-shacache/test.cfg --key ocean.nexedi.com:river:debian:"$KERNEL_VERSION" "timestamp<<Infinity" > "$tmp_image_name"
image_hash=$(sha512sum "$tmp_image_name" | cut -d' ' -f1)
mv "$tmp_image_name" "$image_hash"
echo image_hash = "$image_hash"
mkdir -p /boot/efi/"$RELATIVE_EFI_IMAGE_DIR"
cp "$image_hash" /boot/efi/"$RELATIVE_EFI_IMAGE_DIR"

# Delete $default_bootnum so that there is only one boot option for an image created by this script
efibootmgr -b "$default_bootnum" -B || true

# Set the image as a boot option
efibootmgr -b "$default_bootnum" --create --disk "$EFI_PARTITION_MOUNT_POINT" --label "test_deploy" --loader "${RELATIVE_EFI_IMAGE_DIR}/$image_hash"

echo image path =
echo "${RELATIVE_EFI_IMAGE_DIR}/$image_hash"
